@@ -66,34 +66,70 @@ use Moose;
 use namespace::autoclean;
 
 # ABSTRACT: Read extended MANIFEST file
-our $VERSION = 'v0.3.0'; # VERSION
+our $VERSION = 'v0.3.1'; # VERSION
 
 with 'Dist::Zilla::Role::FileGatherer';
 with 'Dist::Zilla::Role::FileFinder';
-with 'Dist::Zilla::Role::ErrorLogger'  => { -version => 0.005 };
+with 'Dist::Zilla::Role::ErrorLogger'  => { -version => 0.006 }; # need log_errors_in_file
 
-use Path::Tiny;
+use Dist::Zilla::File::OnDisk;
 use List::Util qw{ min max };
+use Path::Tiny;
 use Try::Tiny;
 
 # --------------------------------------------------------------------------------------------------
 
-#pod =attr manifest
+#pod =attr manifest_name
 #pod
 #pod Name of manifest file to read.
 #pod
-#pod C<Str>, read-only, default value is C<MANIFEST>.
+#pod C<Str>, read-only, default value is C<MANIFEST>, C<init_arg> is C<manifest>.
 #pod
 #pod =cut
 
-has manifest => (
+has manifest_name => (
     isa         => 'Str',
     is          => 'ro',
     default     => 'MANIFEST',
+    init_arg    => 'manifest',
 );
 
 # --------------------------------------------------------------------------------------------------
 
+#pod =attr manifest_file
+#pod
+#pod Manifest file as a C<Dist::Zilla> file object (C<Dist::Zilla::File::OnDisk>).
+#pod
+#pod C<Object>, read-only.
+#pod
+#pod =cut
+
+has manifest_file => (
+    isa         => 'Dist::Zilla::File::OnDisk',
+    is          => 'ro',
+    lazy        => 1,
+    builder     => '_build_manifest_file',
+    init_arg    => undef,
+);
+
+sub _build_manifest_file {
+    my ( $self ) = @_;
+    #   Straightforward appoach
+    #       path( $self->manifest_name )
+    #   is incorrect: it works only if the root directory is the current one, which is not always
+    #   true. Following expression for manifest path is correct, but gives absolute (and often too
+    #   long) name:
+    #       path( $self->zilla->root )->child( $self->manifest )
+    #   Let's try to shorten it. Hope this works:
+    #       path( $self->zilla->root )->child( $self->manifest )->relative
+    #   ...until someone changes the current directory...
+    return Dist::Zilla::File::OnDisk->new( {
+        name => path( $self->zilla->root )->child( $self->manifest_name )->relative . '',
+    } );
+};
+
+# --------------------------------------------------------------------------------------------------
+
 #pod =method gather_files
 #pod
 #pod This method fulfills L<Dist::Zilla::Role::FileGatherer> role requirement. It adds files listed in
@@ -121,6 +157,9 @@ sub gather_files {
 #pod This method can be called by other plugins to iterate through files added by C<Manifest::Read>,
 #pod see L</"SYNOPSIS">.
 #pod
+#pod Note: Plugins which remove files from distribution (i. e. plugins which do C<FilePruner> role) do
+#pod not affect result of this method.
+#pod
 #pod =cut
 
 sub find_files {
@@ -150,15 +189,17 @@ has _files => (
 sub _build_files {
     my ( $self ) = @_;
     my $files = [];
+    my @errors;
     my $error = sub {
         my ( $item, $message ) = @_;
-        return $self->log_error( [
+        my $err = sprintf(
             '%s %s at %s line %d.',
-            $item->{ filename }, $message, $self->manifest, $item->{ line }
-        ] );
+            $item->{ filename }, $message, $self->manifest_name, $item->{ line },
+        );
+        push( @errors, $item->{ line } => $err );
+        return $self->log_error( $err );
     };
     foreach my $item ( $self->_parse_lines() ) {
-        # TODO: _show_context.
         -e $item->{ filename } or $error->( $item, 'does not exist' ) and next;
         if ( $item->{ marker } eq '/' ) {
             -d _ or $error->( $item, 'is not a directory' ) and next;
@@ -170,6 +211,9 @@ sub _build_files {
             };
         };
     };
+    if ( @errors ) {
+        $self->log_errors_in_file( $self->manifest_file, @errors );
+    };
     $self->abort_if_error();
     return $files;
 };
@@ -196,17 +240,16 @@ sub _build_lines {
     my ( $self ) = @_;
     my $lines = [];
     try {
-        my $manifest = path( $self->zilla->root )->child( $self->manifest );
-        @$lines = $manifest->lines_utf8( { chomp => 1 } );
+        @$lines = split( "\n", $self->manifest_file->content );
     } catch {
         my $ex = $_;
         if ( blessed( $ex ) and $ex->isa( 'Path::Tiny::Error' ) ) {
-            $self->log_error( [ '%s: %s', $ex->{ file }, $ex->{ err } ] );
+            $self->abort( [ '%s: %s', $ex->{ file }, $ex->{ err } ] );
         } else {
-            $self->log_error( "$ex" );
+            $self->abort( "$ex" );
         };
-        $self->abort();
     };
+    chomp( @$lines );
     return $lines;
 };
 
@@ -243,7 +286,7 @@ my %RE = (
 
 sub _parse_lines {
     my ( $self ) = @_;
-    my $manifest = $self->manifest;         # Shorter name.
+    my $manifest = $self->manifest_name;         # Shorter name.
     my ( %files, @files );
     my @errors;
     my $n = 0;
@@ -279,8 +322,8 @@ sub _parse_lines {
                 $self->log_error( [ '%s at %s line %d', $filename, $manifest, $n ] );
                 $self->log_error( [ '    also listed at %s line %d.', $manifest, $f->{ line } ] );
                 push( @errors,
-                    $n           => 'The file also listed at line ' . $f->{ line },
-                    $f->{ line } => 'The file also listed at line ' . $n,
+                    $n           => 'The file also listed at line ' . $f->{ line } . '.',
+                    $f->{ line } => 'The file also listed at line ' . $n . '.',
                 );
                 next;
             };
@@ -294,14 +337,14 @@ sub _parse_lines {
             push( @files, $file );
             1;
         } or do {
-            $self->log_error( [ 'Syntax error at %s line %d.', $manifest, $n ] );
-            push( @errors, $n => 'Syntax error' );
+            my $error = sprintf( 'Syntax error at %s line %d.', $manifest, $n );
+            $self->log_error( $error );
+            push( @errors, $n => $error );
             next;
         };
     };
     if ( @errors ) {
-        $self->log_error( [ '%s:', $manifest ] );
-        $self->_show_context( $self->_lines, @errors );
+        $self->log_errors_in_file( $self->manifest_file, @errors );
         $self->abort();
     };
     return @files;
@@ -309,74 +352,6 @@ sub _parse_lines {
 
 # --------------------------------------------------------------------------------------------------
 
-has mr_context => (
-    isa         => 'Int',
-    is          => 'ro',
-    default     => 2,
-);
-
-# --------------------------------------------------------------------------------------------------
-
-# TODO: Move it to error logger?
-
-sub _show_context {
-    my ( $self, $text, @notes ) = @_;
-    #   TODO: Chop too long lines?
-    if ( not ref( $text ) ) {
-        $text  = [ split( "\n", $text ) ];
-    };
-    my %notes;
-    while ( @notes ) {
-        my ( $n, $msg ) = splice( @notes, 0, 2 );
-        if ( $n > 0 ) {
-            my $ctx = $self->mr_context;
-            for my $i ( max( $n - $ctx, 1 ) .. min( $n + $ctx, @$text + 0 ) ) {
-                if ( not $notes{ $i } ) {
-                    $notes{ $i } = [];
-                };
-            };
-            push( @{ $notes{ $n } }, $msg );
-        };
-    };
-    my $w        = length( 0 + @$text );        # Width of linenumber column.
-    my $indent = ' ' x 4;
-    my $fline  = $indent . '%0' . $w . 'd: %s';
-    my $fnote  = $indent . ( ' ' x $w ) . '  ^^^ %s ^^^';               # Notice line format.
-    my $fskip  = $indent . ( ' ' x $w ) . '  ...skipped %d lines...';   # "Skipped" notice format.
-    my $last   = 0;                             # Number of the last printed line.
-    my $show_line = sub {
-        my ( $n ) = @_;
-        my $line = $text->[ $n - 1 ];
-        chomp( $line );
-        $self->log_error( [ $fline, $n, $line ] );
-    };
-    my $show_note = sub {
-        my ( $n ) = @_;
-        $self->log_error( [ $fnote, $_ ] ) for @{ $notes{ $n } };
-    };
-    my $show_skip = sub {
-        my ( $n ) = @_;
-        if ( $n > $last + 1 ) {                 # There are skipped line.
-            my $count = $n - $last - 1;         # Number of skipped lines.
-            if ( $count == 1 ) {
-                $show_line->( $n - 1 );         # There is no sense to skip one line.
-            } else {
-                $self->log_error( [ $fskip, $count ] );
-            };
-        };
-    };
-    for my $n ( sort( { $a <=> $b } keys( %notes ) ) ) {
-        $show_skip->( $n );
-        $show_line->( $n );
-        $show_note->( $n );
-        $last = $n;
-    };
-    $show_skip->( @$text + 1 );
-    return;
-};
-
-# --------------------------------------------------------------------------------------------------
-
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -407,7 +382,7 @@ Dist::Zilla::Plugin::Manifest::Read - Read extended MANIFEST file
 
 =head1 VERSION
 
-Version v0.3.0, released on 2015-09-20 21:56 UTC.
+Version v0.3.1, released on 2015-09-23 11:45 UTC.
 
 This is C<Dist::Zilla::Plugin::Manifest::Read> module documentation. Read this if you are going to hack or
 extend C<Dist-Zilla-Plugin-Manifest-Read>, or use it programmatically.
@@ -438,11 +413,17 @@ at the first problem but continue and report multiple errors to user.
 
 =head1 OBJECT ATTRIBUTES
 
-=head2 manifest
+=head2 manifest_name
 
 Name of manifest file to read.
 
-C<Str>, read-only, default value is C<MANIFEST>.
+C<Str>, read-only, default value is C<MANIFEST>, C<init_arg> is C<manifest>.
+
+=head2 manifest_file
+
+Manifest file as a C<Dist::Zilla> file object (C<Dist::Zilla::File::OnDisk>).
+
+C<Object>, read-only.
 
 =head2 _files
 
@@ -474,6 +455,9 @@ to the distribution.
 This method can be called by other plugins to iterate through files added by C<Manifest::Read>,
 see L</"SYNOPSIS">.
 
+Note: Plugins which remove files from distribution (i. e. plugins which do C<FilePruner> role) do
+not affect result of this method.
+
 =head2 _parse_lines
 
 This method parses manifest lines. Each line is parsed separately (there is no line continuation).
